import naas_abi_cli.cli.new.agent as agent  # noqa: F401
import naas_abi_cli.cli.new.integration as integration  # noqa: F401
import naas_abi_cli.cli.new.module as module  # noqa: F401
import naas_abi_cli.cli.new.orchestration as orchestration  # noqa: F401
import naas_abi_cli.cli.new.pipeline as pipeline  # noqa: F401
import naas_abi_cli.cli.new.project as project  # noqa: F401
import naas_abi_cli.cli.new.workflow as workflow  # noqa: F401

from .new import new  # noqa: F401
